#include <iostream.h>
#include "symbol.hpp"

int main ()
{
  SymbolTableClass st;
  SymbolTableEntry *func, *var;
  SymbolTableEntry *entry, *entry1, *entry2;

  st.OpenScope("example");

  func = new FunctionEntry("f", VoidRet);
  st.Insert(func);
  st.OpenScope("f");

  entry = new ParameterEntry("b", ByValue, CharType, 
			     (FunctionEntry *) func,
			     st.GetCurrentScope());
  st.Insert(entry);
  st.CloseScope();

  func = new FunctionEntry("ditto", VoidRet);
  st.Insert(func);
  st.OpenScope("ditto");

  entry = new ParameterEntry("num", ByValue, IntegerType,
			     (FunctionEntry *) func,
			     st.GetCurrentScope());
  st.Insert(entry);

  entry = new ParameterEntry("b", ByValue, CharType,
			     (FunctionEntry *) func,
			     st.GetCurrentScope());
  st.Insert(entry);

  entry = new ParameterEntry("max", ByValue, IntegerType,
			     (FunctionEntry *) func,
			     st.GetCurrentScope());
  st.Insert(entry);

  st.CloseScope();		// ditto

  entry = new VariableEntry("x", IntegerType, st.GetCurrentScope());
  st.Insert(entry);

  entry = new VariableEntry("s", CharType, st.GetCurrentScope());
  st.Insert(entry);

  var = st.Lookup("x", SEARCH_ALL_SCOPES);
  if (var->WhatIs() != VariableEnt)
    cout << "Not an l-value\n";

  entry = new ConstantEntry(3, st.GetCurrentScope());
  st.Insert(entry);

  entry = new ConstantEntry(5, st.GetCurrentScope());
  st.Insert(entry);

  entry = new TemporaryEntry(st.GetCurrentScope(), IntegerType);
  st.Insert(entry);

  if (((VariableEntry *) var)->GetType() != IntegerType)
    cout << "Not of integer type\n";

  func = st.Lookup("ditto", SEARCH_ALL_SCOPES);
  if (func->WhatIs() != FunctionEnt ||
      ((FunctionEntry *) func)->GetReturnType() != VoidRet)
    cout << "Not a procedure\n";

  entry1 = st.Lookup("x", SEARCH_ALL_SCOPES);
  entry2 = ((FunctionEntry *) func)->GetArgument(1);
  if (((VariableEntry *) entry1)->GetType() !=
      ((VariableEntry *) entry2)->GetType())
    cout << "Not of the same type\n";

  entry1 = st.Lookup("s", SEARCH_ALL_SCOPES);
  entry2 = ((FunctionEntry *) func)->GetArgument(2);

  if (((VariableEntry *) entry1)->GetType() !=
      ((VariableEntry *) entry2)->GetType())
    cout << "Not of the same type\n";

  entry1 = st.Lookup("20", SEARCH_ALL_SCOPES);
  entry1 = new ConstantEntry(20, st.GetCurrentScope());
  st.Insert(entry1);
  entry2 = ((FunctionEntry *) func)->GetArgument(3);

  if (((VariableEntry *) entry1)->GetType() !=
      ((VariableEntry *) entry2)->GetType())
    cout << "Not of the same type\n";

  func = st.Lookup("f", SEARCH_ALL_SCOPES);
  if (func->WhatIs() != FunctionEnt ||
      ((FunctionEntry *) func)->GetReturnType() != VoidRet)
    cout << "Not a procedure\n";

  entry1 = st.Lookup("s", SEARCH_ALL_SCOPES);
  entry2 = ((FunctionEntry *) func)->GetArgument(1);
  if (((VariableEntry *) entry1)->GetType() !=
      ((VariableEntry *) entry2)->GetType())
    cout << "Not of the same type\n";

  st.CloseScope();

  return 0;
}
